import {
  OAuth2RefreshTokenAdapter,
  OAuth2RefreshToken,
} from '@icynet/oauth2-provider';
import { OAuth2TokenType } from 'src/modules/objects/oauth2-token/oauth2-token.entity';
import { Injectable } from '@nestjs/common';
import { OAuth2ClientService } from 'src/modules/objects/oauth2-client/oauth2-client.service';
import { UserService } from 'src/modules/objects/user/user.service';
import { TokenService } from 'src/modules/utility/services/token.service';
import { OAuth2TokenService } from 'src/modules/objects/oauth2-token/oauth2-token.service';
import { FormUtilityService } from 'src/modules/utility/services/form-utility.service';

@Injectable()
export class RefreshTokenAdapter implements OAuth2RefreshTokenAdapter {
  constructor(
    private readonly clientService: OAuth2ClientService,
    private readonly userService: UserService,
    private readonly token: TokenService,
    private readonly tokenService: OAuth2TokenService,
    private readonly form: FormUtilityService,
  ) {}

  invalidateOld = false;

  async create(
    userId: number,
    clientId: string,
    scope: string | string[],
  ): Promise<string> {
    const client = await this.clientService.getById(clientId);
    const user = await this.userService.getById(userId);
    const accessToken = this.token.generateString(64);

    // Standardize scope value
    const scopes = (
      !Array.isArray(scope) ? this.form.splitScope(scope) : scope
    ).join(' ');

    const expiresAt = new Date(Date.now() + 3.154e7 * 1000);

    this.tokenService.insertToken(
      accessToken,
      OAuth2TokenType.REFRESH_TOKEN,
      client,
      scopes,
      expiresAt,
      user,
    );

    return accessToken;
  }

  async fetchByToken(
    token: string | OAuth2RefreshToken,
  ): Promise<OAuth2RefreshToken> {
    const findBy = typeof token === 'string' ? token : token.token;
    const find = await this.tokenService.fetchByToken(
      findBy,
      OAuth2TokenType.REFRESH_TOKEN,
    );

    if (!find) {
      return null;
    }

    return {
      ...find,
      client_id: find.client.client_id,
      user_id: find.user.id,
    };
  }

  async removeByUserIdClientId(
    userId: number,
    clientId: string,
  ): Promise<boolean> {
    const find = await this.tokenService.fetchByUserIdClientId(
      userId,
      clientId,
      OAuth2TokenType.REFRESH_TOKEN,
    );

    await this.tokenService.remove(find);

    return true;
  }

  async removeByRefreshToken(token: string): Promise<boolean> {
    const find = await this.tokenService.fetchByToken(
      token,
      OAuth2TokenType.REFRESH_TOKEN,
    );

    await this.tokenService.remove(find);

    return true;
  }

  getUserId(code: OAuth2RefreshToken): number {
    return code.user_id as number;
  }

  getClientId(code: OAuth2RefreshToken): string {
    return code.client_id as string;
  }

  getScope(code: OAuth2RefreshToken): string {
    return code.scope;
  }
}
